'''
The zodbpickle.pickle module exposes the standard behavior of
the pickle module.
This is backward compatible, but has the effect that by default,
on Python3 you get the fast implementation, while on Python2
you get the slow implementation.

This module is a version that always exposes the slow implementation
of pickling and avoids the need to explicitly touch internals.


Note: We are intentionally using "import *" in this context.
The imported modules define an __all__ variable, which contains
all the names that it wants to export.
So this is a rare case where 'import *' is exactly the right thing to do.
'''

import sys

import zodbpickle.pickle_3 as p


# undo the replacement with fast versions
p.Pickler, p.Unpickler = p._Pickler, p._Unpickler
p.dump, p.dumps, p.load, p.loads = p._dump, p._dumps, p._load, p._loads
del p

# isort: off
# pick up all names that the module defines
from .pickle_3 import *  # noqa: E402 module level import not at top of file


# do not share the globals with a fast version
del sys.modules['zodbpickle.pickle_3']



##############################################################################
#
# Copyright (c) 2022 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import struct
import unittest


try:
    import zodbpickle._pickle  # noqa: try to load a C module for side effects
except ModuleNotFoundError:    # pragma: no cover
    pass


class TestFloatingPoint(unittest.TestCase):

    def test_no_fast_math_optimization(self):
        # Building with -Ofast enables -ffast-math, which sets certain FPU
        # flags that can cause breakage elsewhere.  A library such as BTrees
        # has no business changing global FPU flags for the entire process.
        zero_bits = struct.unpack("!Q", struct.pack("!d", 0.0))[0]
        next_up = zero_bits + 1
        smallest_subnormal = struct.unpack("!d", struct.pack("!Q", next_up))[0]
        self.assertNotEqual(smallest_subnormal, 0.0)



import os
import platform


py_impl = getattr(platform, 'python_implementation', lambda: None)
_is_pypy = py_impl() == 'PyPy'
_is_jython = py_impl() == 'Jython'
_is_pure = int(os.environ.get('PURE_PYTHON', '0'))
